@@ -2,7 +2,8 @@
 # archiver.py
 #
 
-import os, sys, socket, re, time, smtplib, calendar, StringIO, logging, logging.handlers, locale
+import os, sys, socket, re, time, smtplib, calendar, StringIO, logging, logging.handlers, locale, gzip
+import urllib, urllib2
 from datetime import datetime, timedelta
 import email
 from email.Utils import parseaddr, formataddr, formatdate
@@ -10,7 +11,6 @@ from email.MIMEMultipart import MIMEMultipart
 from email.MIMEText import MIMEText
 from email.MIMEBase import MIMEBase
 
-
 import cron
 
 atom=r"[a-zA-Z0-9_#\$&'*+/=?\^`{}~|\-]+"
@@ -29,6 +29,17 @@ email_addressRE=re.compile('^'+postfix_restricted_rfc2822_email_address+'$')
 valid_ipRE=re.compile('^([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$')
 valid_hostnameRE=re.compile('^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]*[a-zA-Z0-9])\\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\\-]*[A-Za-z0-9])$')
 
+# from FormEncode
+url_re = re.compile(r'''
+    ^(http|https)://
+    (?:[%:\w]*@)?                           # authenticator
+    (?P<domain>[a-z0-9][a-z0-9\-]{1,62}\.)* # (sub)domain - alpha followed by 62max chars (63 total)
+    (?P<tld>[a-z]{2,})                      # TLD
+    (?::[0-9]+)?                            # port
+    # files/delims/etc
+    (?P<path>/[a-z0-9\-\._~:/\?#\[\]@!%\$&\'\(\)\*\+,;=]*)?
+    $
+''', re.I | re.VERBOSE)
 
 boolean={ 'on':True, 'yes':True, 'true':True, '1':True, 1:True, 'off':False, 'no':False, 'false':False, '0':False, 0:False}
 
@@ -54,6 +65,13 @@ def check_mail_config(job):
         errors['mail']='boolean must have value in (on, yes, true, 1, off, no, false and 0)'
     else:
         job['mail']=mail
+
+    magikmon=job.get('magikmon', '')
+    if magikmon and not url_re.match(magikmon):
+        errors['magikmon']='not a valid URL'
+        
+    job['magikmon']=magikmon
+
     
     if mail:
         try:
@@ -94,6 +112,8 @@ def check_mail_config(job):
             else:
                 job['smtp_port']=port
             
+    if mail or magikmon:
+
         sender=job.get('sender', None)
         if not sender:
             errors['sender']='option mandatory'
@@ -114,37 +134,55 @@ def check_mail_config(job):
             else:
                 job['recipients']=recipients
 
+        attachment_gzip=job['attachment_gzip']
+        try:
+            size=int(attachment_gzip)
+        except Exception:
+            errors['attachment_gzip']='must be an integer'
+        else:
+            if not (0<size and size<=10240):
+                errors['attachment_gzip']='must be an integer between 1 and 10240'
+            else:
+                job['attachment_gzip']=size*1024 # in K
+
         attachment_size=job['attachment_size']
         try:
             size=int(attachment_size)
         except Exception:
             errors['attachment_size']='must be an integer'
         else:
-            if not (0<size and size<100000):
-                errors['smtp_port']='must be an integer between 1 and 100000'
+            if not (0<size and size<=10240):
+                errors['attachment_size']='must be an integer between 1 and 10240'
             else:
-                job['attachment_size']=size
-                
+                job['attachment_size']=size*1024 # in K
+
+
     return errors
 
+# ---------------------------------------------------------------------------
+def _list_dir(target_dir, log):
+    total_size=0
+    dir_out=u'== directory %s\r\n  epoch   |          time          |      size     |    filename\r\n' % target_dir
+    for filename in os.listdir(target_dir): # if target_dir is unicode, return os.listdir return unicode
+        full_filename=os.path.join(target_dir, filename)
+        stat=os.stat(full_filename)
+        if os.path.isdir(full_filename):
+            size='<DIR>'
+        else:
+            size='%15d' % stat.st_size
+            total_size+=stat.st_size
+        dir_out+='%d %s %15s %s\r\n' % (stat.st_mtime, time.ctime(stat.st_mtime), size, filename)
+        
+    return dir_out, total_size
 
 # ---------------------------------------------------------------------------
 def list_dir(target_dir, log):
-    total_size=0
-    try: 
-        dir_out=u'== directory %s\r\n  epoch   |          time          |      size     |    filename\r\n' % target_dir
-        for filename in os.listdir(target_dir): # if target_dir is unicode, return os.listdir return unicode
-            full_filename=os.path.join(target_dir, filename)
-            stat=os.stat(full_filename)
-            if os.path.isdir(full_filename):
-                size='<DIR>'
-            else:
-                size='%15d' % stat.st_size
-                total_size+=stat.st_size
-            dir_out+='%d %s %15s %s\r\n' % (stat.st_mtime, time.ctime(stat.st_mtime), size, filename)
+    try:
+        dir_out, total_size=_list_dir(target_dir, log) 
     except Exception, e:
         log.error('listing target directory: %s', e)
         dir_out='error listing directory "%s": %s\r\n' % (target_dir, e)
+        total_size=0
         
     return dir_out, total_size
 
@@ -195,18 +233,16 @@ def quoted_string_list(st):
     return lst
 
 # ---------------------------------------------------------------------------
-def sendmail(job, subject, text, attachements, log=None):
+def sendmail(job, subject, text, attachments, log):
+
+    if not job['mail'] and not job['magikmon']:
+        return None
 
     sender=job['sender']
     recipients=job['recipients']
-    smtp_host=job['smtp_host']
-    smtp_port=job['smtp_port']
-    smtp_mode=job['smtp_mode']
-    smtp_login=job['smtp_login']
-    smtp_password=job['smtp_password']
     attachment_size=job['attachment_size']
+    attachment_gzip=job['attachment_gzip']
 
-    attachment_size*=1024 # in K bytes
     msg=MIMEMultipart()
     msg.preamble='' # This line is not visible on mime enable MUA
     msg.epilogue=''
@@ -219,96 +255,138 @@ def sendmail(job, subject, text, attachements, log=None):
     core=MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
     msg.attach(core)
     
-    for filename, target, data, maintype, subtype, charset in attachements:
+    for filename, target, data, maintype, subtype, charset in attachments:
         
         if not data and target and os.path.exists(target):
             data=open(target, 'rb').read(attachment_size)
 
         if data:
-            data=data[:attachment_size]
+            if len(data)>attachment_gzip:
+                log.info('gzip attachment %s', filename)
+                file_out=StringIO.StringIO()
+                zfile=gzip.GzipFile(filename, 'wb', 9, file_out)
+                if maintype=='text':
+                    # all txt gzip file are encode in UTF8 to avoid the loose of the charset 
+                    data=data.decode(charset).encode('utf-8')
+                zfile.write(data)
+                zfile.close()
+                data=file_out.getvalue()
+                file_out.close()
+                filename+='.gz'
+                maintype, subtype='application', 'x-gzip'
+
+            if len(data)>attachment_size:
+                log.warning('attachment %s too big, file skipped', filename)
+                continue
+
             #maintype, subtype = 'application', 'octet-stream'
             if maintype=='text':
-                attachement=MIMEText(data, subtype, charset)
+                attachment=MIMEText(data, subtype, charset)
             else:
-                attachement=MIMEBase(maintype, subtype)
-                attachement.set_payload(data) #, charset) #dont use the charset, here !
-                email.Encoders.encode_base64(attachement)
+                attachment=MIMEBase(maintype, subtype)
+                attachment.set_payload(data) #, charset) #dont use the charset, here !
+                email.Encoders.encode_base64(attachment)
     
-        attachement.add_header('Content-Disposition', 'attachment', filename=filename)
-        msg.attach(attachement)
+            attachment.add_header('Content-Disposition', 'attachment', filename=filename)
+            msg.attach(attachment)
     
     body=msg.as_string()
-    rcpt=[]
-    try:
-        if smtp_mode=='ssl':
-            smtp=smtplib.SMTP_SSL(smtp_host, smtp_port)
-        else:
-            smtp=smtplib.SMTP(smtp_host, smtp_port)
-            if smtp_mode=='tls':
-                smtp.starttls()
-                
-        if smtp_login and smtp_password:
-            smtp.login(smtp_login, smtp_password)
 
-        ret=smtp.sendmail(sender, recipients, body)
-        smtp.quit()
-    except (socket.error, ), e:
-        errmsg='server %s:%s not responding' % (smtp_host, smtp_port)
-    except smtplib.SMTPAuthenticationError, e:
-        errmsg='authentication error: %s' % (e, )
-    except smtplib.SMTPRecipientsRefused, e:
-        # code, errmsg=e.recipients[recipient_addr]
-        errmsg='recipients refused: '+', '.join(e.recipients.keys())
-    except smtplib.SMTPSenderRefused, e:
-        # e.sender, e.smtp_code, e.smtp_error
-        errmsg='sender refused: %s' % (e.sender, )
-    except smtplib.SMTPDataError, e:
-        errmsg='SMTP protocol mismatch'
-    except smtplib.SMTPHeloError, e:
-        errmsg="server didn't reply properly to the HELO greeting"
-    except smtplib.SMTPException, e:
-        errmsg=str(e)
-    except Exception, e:
-        errmsg=str(e)
-    else:
-        rcpt=recipients[:]
-        if ret:
-            errmsg='recipients refused: '+', '.join(ret.keys())        
-            for r in ret.keys():
-                rcpt.remove(r)
+    errmsg=''     
+    if job['mail']:
+        smtp_host=job['smtp_host']
+        smtp_port=job['smtp_port']
+        smtp_mode=job['smtp_mode']
+        smtp_login=job['smtp_login']
+        smtp_password=job['smtp_password']
+    
+        rcpt=[]
+        try:
+            if smtp_mode=='ssl':
+                smtp=smtplib.SMTP_SSL(smtp_host, smtp_port)
+            else:
+                smtp=smtplib.SMTP(smtp_host, smtp_port)
+                if smtp_mode=='tls':
+                    smtp.starttls()
+                    
+            if smtp_login and smtp_password:
+                # login and password must be encoded because HMAC used in CRAM_MD5 require non unicode string
+                smtp.login(smtp_login.encode('utf-8'), smtp_password.encode('utf-8'))
+    
+            ret=smtp.sendmail(sender, recipients, body)
+            smtp.quit()
+        except (socket.error, ), e:
+            errmsg='server %s:%s not responding' % (smtp_host, smtp_port)
+        except smtplib.SMTPAuthenticationError, e:
+            errmsg='authentication error: %s' % (e, )
+        except smtplib.SMTPRecipientsRefused, e:
+            # code, errmsg=e.recipients[recipient_addr]
+            errmsg='recipients refused: '+', '.join(e.recipients.keys())
+        except smtplib.SMTPSenderRefused, e:
+            # e.sender, e.smtp_code, e.smtp_error
+            errmsg='sender refused: %s' % (e.sender, )
+        except smtplib.SMTPDataError, e:
+            errmsg='SMTP protocol mismatch: %s' % (e, )
+        except smtplib.SMTPHeloError, e:
+            errmsg="server didn't reply properly to the HELO greeting: %s" % (e, )
+        except smtplib.SMTPException, e:
+            errmsg='SMTP error: %s' % (e, )
+        except Exception, e:
+            errmsg=str(e)
+            log.exception('unexpected error sending mail: %s', e)
         else:
-            errmsg=''     
-
+            rcpt=recipients[:]
+            if ret:
+                errmsg='recipients refused: '+', '.join(ret.keys())        
+                for r in ret.keys():
+                    rcpt.remove(r)
+    
+            if log:
+                log.info('mail sent to: %s', ', '.join(rcpt))
+    
         if log:
-            log.info('mail sent to: %s', ', '.join(rcpt))
+            if errmsg:
+                log.error('sending mail: %s', errmsg)
 
-    if log:
-        if errmsg:
-            log.error('sending mail: %s', errmsg)
+            
+    if job['magikmon'] and job['program']!='esxmon':
+        data=urllib.urlencode(dict(body=body))
+        try:
+            try:
+                urllib2.urlopen(job['magikmon'], data, timeout=30)
+            except TypeError:
+                # python 2.4
+                urllib2.urlopen(job['magikmon'], data)
+        except Exception, e:
+            log.exception('error sending mail to MagiKmon url: %s (%s)', job['magikmon'], e)
+            if not errmsg:
+                errmsg='error sending mail to MagiKmon url'
+        else:
+            log.debug('mail sent to MagiKmon url: %s', job['magikmon'])
 
     return errmsg
 
 
 # ---------------------------------------------------------------------------
 def send_mail_report(job, result, manager):
-    if not job['mail']:
+    if not job['mail'] and not job['magikmon']:
         return None
     
-    backup_status, status, attachements=result
+    backup_status, status, attachments=result
 
     msg_body=job['msg_header']+'\n'+status
     subject='MKBACKUP BACKUP %s %s' % (backup_status, job['name'])
     
-    attachements.extend(manager.attachements)
-    attachements.append(('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'))
+    attachments.extend(manager.attachments)
+    attachments.append(('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'))
     log_output=manager.stop_logging()
-    attachements.append(('logging.txt', None, log_output.encode('utf-8'), 'text', 'plain', 'utf-8'))                       
+    attachments.append(('logging.txt', None, log_output.encode('utf-8'), 'text', 'plain', 'utf-8'))                       
     
     for line in status.split('\r\n'):
         if line:
            manager.log.info('    %s', line)
     
-    return sendmail(job, subject, msg_body, attachements, manager.log)
+    return sendmail(job, subject, msg_body, attachments, manager.log)
 
 
 # ---------------------------------------------------------------------------
@@ -462,6 +540,16 @@ class MyMemoryHandler(logging.handlers.MemoryHandler):
             self.buffer=self.buffer[-self.capacity:]
             
         return False
+
+# -------------------------------------------------------------------------
+def ErrorDecode(exception, encoding='utf-8'):
+    """This is the equivalent of my windows.WindowsErrorDecode but for 
+    non windows OS"""
+    st=str(exception)
+    try:
+        return st.decode(encoding)
+    except UnicodeDecodeError:
+        return st.decode('ascii', 'replace')
    
 
 # ---------------------------------------------------------------------------
@@ -474,11 +562,12 @@ class Manager:
         self.log=log
         self.default_encoding=locale.getdefaultlocale()[1]
         self.console_encoding=sys.stdout.encoding
+        self.formater=logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S')
         
     def start_logging(self):
         self.string_log=StringIO.StringIO()
         self.stream_log=logging.StreamHandler(self.string_log)
-        self.stream_log.setFormatter(logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S'))
+        self.stream_log.setFormatter(self.formater)
         self.memory_log=MyMemoryHandler(self.memory_log_capacity, logging.INFO, self.stream_log)
         self.log.addHandler(self.memory_log)
 
@@ -519,3 +608,37 @@ class Archiver:
             self._hidden_password.append(password)
     
     # -----------------------------------------------------------------------
+
+# =====================================================================
+class UnixRegistry:
+
+    directory='.mksbackup'
+    
+    def __init__(self, master_key):
+        path=os.path.join(os.path.expanduser('~'), self.directory)
+        if not os.path.isdir(path):
+            os.mkdir(path)
+        self.filename=os.path.join(path, master_key)
+        
+    def LoadValues(self, keys=[]):
+        values=dict()
+        try:
+            for line in open(self.filename).read().split('\n'):
+                line=line.strip()
+                if not line: continue
+                key, value=line.split('=', 1)
+                values[key]=value
+        except IOError:
+            pass
+        return values    
+                
+    def SaveValues(self, values):
+        data=""
+        for k, v in values.iteritems():
+            data+='%s=%s\n' % (k, str(v))
+        filedata=open(self.filename, "w")
+        filedata.write(data)
+        filedata.close()
+        
+    def Close(self):
+        pass
\ No newline at end of file
