@@ -35,7 +35,12 @@ from archiver import boolean, check_mail_config, Destinations, sendmail, Manager
 
 import cron
 
-__version__='0.9.6'
+if sys.platform in ('win32', ):
+    from windows import WindowsErrorDecode as ErrorDecode
+else:
+    from archiver import ErrorDecode
+
+__version__='0.9.10'
 last_version_url='http://www.magikmon.com/download/mkbackup_last.txt'
 
 def gen_archiver(name):
@@ -49,6 +54,9 @@ def gen_archiver(name):
     elif name=='ghettovcb':
         import ghettovcb
         return ghettovcb.Ghettovcb
+    elif name=='esxmon':
+        import esxmon
+        return esxmon.ESXMon
     elif name=='wbadmin':
         import wbadmin
         return wbadmin.Wbadmin
@@ -99,22 +107,26 @@ def mail_exception(start, job, extra, manager, subject):
     status+='end_epoch=%d\r\n' % end
     status+='runtime_error=%r\r\n' % sys.exc_info()[1]
     
-    # TODEL: subject='MKBACKUP BACKUP ERR %s' % (job['name'], )
     msg_body=job['msg_header']
     msg_body+='\nUnexpected error, read "traceback.txt" for more\n\n'
     msg_body+='\n%s\n' % status
     msg_body+=extra+'\n'
 
-    tb=StringIO.StringIO()
-    traceback.print_exc(None, tb)
+    try:
+        tb=StringIO.StringIO()
+        exctype, value = sys.exc_info()[:2]
+        traceback.print_exc(None, tb)
+        tb_st=tb.getvalue()
+        tb.close()
+    except Exception, e:
+        tb_st='Traceback failed: %r' % e
     
-    attachements=manager.attachements[:]
-    attachements+=[ # the type, subtype and coding is not used
-           ('traceback.txt', None, tb.getvalue().encode('utf-8'), 'text', 'plain', 'utf-8'),
+    attachments=manager.attachments[:]
+    attachments+=[ # the type, subtype and coding is not used
+           ('traceback.txt', None, tb_st.decode('ascii', 'replace'), 'text', 'plain', 'utf-8'), # ascii is already utf8
            ('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'),
     ]
-    tb.close()
-    errmsg=sendmail(job, subject, msg_body, attachements, manager.log)
+    errmsg=sendmail(job, subject, msg_body, attachments, manager.log)
 
 
 class MyFormatter(logging.Formatter):
@@ -124,6 +136,8 @@ class MyFormatter(logging.Formatter):
  
     def format(self, record):
         t=logging.Formatter.format(self, record)
+        return t
+        # ASX
         if isinstance(t, unicode):
             t=t.encode(self.encoding, 'replace')
         return t
@@ -143,14 +157,14 @@ def main():
                       '\tcommand in "backup", "check", "checkmail"')
     
     parser.add_option("-c", "--config", dest="config", default='mkbackup.ini', help="use another ini file", metavar="configfile")
-    parser.add_option("-d", "--debug", dest="debug", action="store_true", default=False, help="switch to debug level")
-    parser.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="write log to the terminal")
+    parser.add_option("-d", "--debug", dest="debug", action="store_true", default=False, help="switch to debug level, increase verbosity")
+    parser.add_option("-v", "--verbose", dest="verbose", action="store_true", default=True, help="write logging to the terminal (default)")
+    parser.add_option("-q", "--quiet", dest="verbose", action="store_false", default=True, help="don't write logging to the terminal")
     parser.add_option("-l", "--logfile", dest="logfile", default='mkbackup.log', help="log to this file", metavar="logfile")
     parser.add_option("-e", "--encoding", dest="encoding", default='auto', help="configuration file encoding", metavar="encoding")
     
     cmd_options, cmd_args=parser.parse_args(sys.argv)
     
-    print 'logging in %s' % cmd_options.logfile
     root_handler=logging.handlers.RotatingFileHandler(cmd_options.logfile, 'a', 1024**2, 2)
     root_handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S'))
     log=logging.getLogger()
@@ -173,7 +187,7 @@ def main():
         log.addHandler(console)
     
     
-    log.info('start version=%s cmd=%r', __version__, sys.argv)
+    log.info(u'start version=%s cmd=%r', __version__, sys.argv)
     
     if len(cmd_args)<2:
         parser.error('no "command" set')
@@ -183,34 +197,16 @@ def main():
         parser.error('invalid command "%s"' % command)
     
     command=command.lower()
-    
+    if command=='backup':
+        command_display='BACKUP'
+    else:
+        command_display='CONFIG'
+        
     job_list=cmd_args[2:]
     
     if not job_list:
         parser.error('no "job"')
     
-    
-    #
-    # load the configuration file
-    #
-    config_default=dict(verify='no',
-                        update_check='yes',
-                        smtp_host='127.0.0.1',
-                        smtp_mode='normal',
-                        night_shift='yes',
-                        attachment_size='100',
-                        )
-    
-    try:
-        config_file=open(cmd_options.config, 'r')
-    except IOError, e:
-        log.error('error reading configuration file "%s": %s', cmd_options.config, e)
-        sys.exit(1)
-    
-    config_text=config_file.read()
-    config_file.close()
-    config_file=None
-    
     # define a default_encoding
     default_encoding=locale.getdefaultlocale()[1]
     console_encoding=sys.stderr.encoding
@@ -228,6 +224,28 @@ def main():
             console_encoding='cp850'
         else:
             console_encoding='utf-8'
+
+    #
+    # load the configuration file
+    #
+    config_default=dict(verify='no',
+                        update_check='yes',
+                        smtp_host='127.0.0.1',
+                        smtp_mode='normal',
+                        night_shift='yes',
+                        attachment_size='1000',
+                        attachment_gzip='100',
+                        )
+    
+    try:
+        config_file=open(cmd_options.config, 'r')
+    except IOError, e:
+        log.error('error reading configuration file "%s": %s', cmd_options.config, ErrorDecode(e, default_encoding))
+        sys.exit(1)
+    
+    config_text=config_file.read()
+    config_file.close()
+    config_file=None
     
     if cmd_options.encoding=='auto':
         # try to guess the file encoding  
@@ -252,14 +270,14 @@ def main():
         try:
             config_file=codecs.open(cmd_options.config, 'r', cmd_options.encoding)        
         except LookupError, e:
-            log.error('error reading configuration file %s: %s', cmd_options.config, e)
+            log.error('error reading configuration file %s: %s', cmd_options.config, ErrorDecoding(e, default_encoding))
             sys.exit(1)
     
     config=ConfigParser.RawConfigParser(config_default)
     try:
         config.readfp(config_file)
     except (IOError, UnicodeDecodeError), e:
-        log.error('error reading configuration file %s: %s', cmd_options.config, e)
+        log.error('error reading configuration file %s: %s', cmd_options.config, ErrorDecoding(e, default_encoding))
         sys.exit(1)
     
     up2date, up2date_msg='unk', ''
@@ -304,8 +322,10 @@ def main():
     manager=Manager(platform.node(), log)
     manager.default_encoding=default_encoding
     manager.console_encoding=console_encoding
+    manager.formater=MyFormatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S', encoding='utf-8')
+
     manager.sanitized_config_file_content=sanitized_config_file_content
-    manager.attachements=[ ('config.ini', None, manager.sanitized_config_file_content.encode(cmd_options.encoding), 'text', 'plain', cmd_options.encoding), ] 
+    manager.attachments=[ ('config.ini', None, manager.sanitized_config_file_content.encode(cmd_options.encoding), 'text', 'plain', cmd_options.encoding), ] 
     
     for job_name in job_list:
     
@@ -336,9 +356,13 @@ def main():
         try:
             errors, warnings, extra=arch.load(job, manager)
         except Exception, e:
-            log.exception('loading job %s by program %s', job_name, program)
+            try:
+                log.exception('loading job %s by program %s', job_name, program)
+            except UnicodeDecodeError:
+                log.error('loading job %s by program %s: %s', job_name, program, ErrorDecoding(e, default_encoding))
+                
             if job['mail'] and not mail_config_errors:
-                subject='MKBACKUP CONFIG ERR %s ' % (job['name'], )
+                subject='MKBACKUP %s ERR %s ' % (command_display, job['name'], )
                 mail_exception(start, job, '', manager, subject)
             continue
     
@@ -355,16 +379,16 @@ def main():
             msg_body+='Errors in section: %s\n\n' % job['name']
             log.error('Errors in section: %s', job['name'])
             for k, v in errors.iteritems():
-                log.error('%s: %s', k, v)
-                msg_body+='%s\n    %s\n\n' % (k, v)
+                log.error('\tparameter "%s" : %s', k, v)
+                msg_body+='       parameter "%s" : %s\n\n' % (k, v)
     
             for k, v in warnings.iteritems():
                 log.warning('%s: %s', k, v)
                 msg_body+='%s\n    %s\n\n' % (k, v)
                     
             if job['mail'] and not mail_config_errors and not command=='check':
-                subject='MKBACKUP CONFIG ERR %s ' % (job['name'], )
-                errmsg=sendmail(job, subject, msg_body, manager.attachements, manager.log)
+                subject='MKBACKUP %s ERR %s ' % (command_display, job['name'], )
+                errmsg=sendmail(job, subject, msg_body, manager.attachments, manager.log)
     
             continue
     
@@ -401,8 +425,8 @@ def main():
                         
             msg_body='%s\n\n%s\n' % (msg_body, extra)
             if job['mail'] and command=='checkmail':
-                subject='MKBACKUP CONFIG OK %s ' % (job['name'], )
-                errmsg=sendmail(job, subject, msg_body, manager.attachements, manager.log)
+                subject='MKBACKUP %s OK %s ' % (command_display, job['name'], )
+                errmsg=sendmail(job, subject, msg_body, manager.attachments, manager.log)
             continue 
     
         #print 'destination', destination
@@ -417,7 +441,11 @@ def main():
             result=arch.run(command, job, manager)
         except Exception, e:
             exit_code=2
-            log.exception('running command=%s job=%s archiver=%s', command, job_name, program)
+            try:
+                log.exception('running command=%s job=%s archiver=%s', command, job_name, program)
+            except UnicodeDecodeError:
+                log.error('running command=%s job=%s archiver=%s : %s', command, job_name, program, ErrorDecoding(e, default_encoding))
+
             if job['mail']:
                 subject='MKBACKUP BACKUP ERR %s ' % (job['name'], )
                 mail_exception(start, job, extra, manager, subject)
@@ -430,10 +458,7 @@ def main():
                 if result[1]!='OK':
                     exit_code=max(exit_code, 1)
     
-                if not job['mail']:
-                    log.info('mail is disabled')
-                else:
-                    errmsg=send_mail_report(job, result, manager)
+                errmsg=send_mail_report(job, result, manager)
     
     return exit_code
     
